"use strict";

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = createClient;

var _request = _interopRequireDefault(require("./request"));

var _debug = _interopRequireDefault(require("debug"));

var _interpreter = require("./interpreter");

var _defaults = _interopRequireDefault(require("./defaults"));

var _jwtDecode2 = _interopRequireDefault(require("jwt-decode"));

var _time = _interopRequireDefault(require("./time"));

var _ = _interopRequireWildcard(require("./lodash"));

var _constants = require("./constants");

var _errors = require("./errors");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _createForOfIteratorHelper(o, allowArrayLike) { var it; if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = o[Symbol.iterator](); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it["return"] != null) it["return"](); } finally { if (didErr) throw err; } } }; }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

var debug = (0, _debug["default"])('craft-ai:client');

function resolveAfterTimeout(timeout) {
  return new Promise(function (resolve) {
    return setTimeout(function () {
      return resolve();
    }, timeout);
  });
} // A very simple regex, helps detect some issues.


var SIMPLE_HTTP_URL_REGEX = /^https?:\/\/.*$/;

function isUrl(url) {
  return SIMPLE_HTTP_URL_REGEX.test(url);
}

var isUnvalidId = function isUnvalidId(id) {
  return !_.isUndefined(id) && !_constants.AGENT_ID_ALLOWED_REGEXP.test(id);
};

var isUnvalidConfiguration = function isUnvalidConfiguration(configuration) {
  return _.isUndefined(configuration) || !_.isObject(configuration);
};

var areUnvalidOperations = function areUnvalidOperations(operations) {
  return _.isUndefined(operations) || !_.isArray(operations);
};

var isInvalidFilter = function isInvalidFilter(filter) {
  return !_.isArray(filter) || filter.some(function (agentName) {
    return isUnvalidId(agentName);
  });
};

function checkBulkParameters(bulkArray) {
  if (_.isUndefined(bulkArray)) {
    throw new _errors.CraftAiBadRequestError('Bad Request, unable to use bulk functionalities without list provided.');
  }

  if (!_.isArray(bulkArray)) {
    throw new _errors.CraftAiBadRequestError('Bad Request, bulk inputs should be provided within an array.');
  }

  if (!bulkArray.length) {
    throw new _errors.CraftAiBadRequestError('Bad Request, the array containing bulk inputs is empty.');
  }
}

function createClient(tokenOrCfg) {
  var cfg = _.defaults({}, _.isString(tokenOrCfg) ? {
    token: tokenOrCfg
  } : tokenOrCfg, _defaults["default"]); // Initialization check


  if (!_.has(cfg, 'token') || !_.isString(cfg.token)) {
    throw new _errors.CraftAiBadRequestError('Bad Request, unable to create a client with no or invalid token provided.');
  }

  try {
    var _jwtDecode = (0, _jwtDecode2["default"])(cfg.token),
        owner = _jwtDecode.owner,
        platform = _jwtDecode.platform,
        project = _jwtDecode.project; // Keep the provided values


    cfg.owner = cfg.owner || owner;
    cfg.project = cfg.project || project;
    cfg.url = cfg.url || platform;
  } catch (e) {
    throw new _errors.CraftAiCredentialsError();
  }

  if (!_.has(cfg, 'url') || !isUrl(cfg.url)) {
    throw new _errors.CraftAiBadRequestError('Bad Request, unable to create a client with no or invalid url provided.');
  }

  if (!_.has(cfg, 'project') || !_.isString(cfg.project)) {
    throw new _errors.CraftAiBadRequestError('Bad Request, unable to create a client with no or invalid project provided.');
  } else {
    var splittedProject = cfg.project.split('/');

    if (splittedProject.length >= 2) {
      cfg.owner = splittedProject[0];
      cfg.project = splittedProject[1];
    }
  }

  if (!_.has(cfg, 'owner') || !_.isString(cfg.owner)) {
    throw new _errors.CraftAiBadRequestError('Bad Request, unable to create a client with no or invalid owner provided.');
  }

  if (cfg.proxy != null && !isUrl(cfg.proxy)) {
    throw new _errors.CraftAiBadRequestError('Bad Request, unable to create a client with an invalid proxy url provided.');
  }

  debug("Creating a client instance for project '".concat(cfg.owner, "/").concat(cfg.project, "' on '").concat(cfg.url, "'."));
  var request = (0, _request["default"])(cfg); // 'Public' attributes & methods

  var instance = {
    cfg: cfg,
    createAgent: function createAgent(configuration) {
      var id = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : undefined;

      if (isUnvalidConfiguration(configuration)) {
        return Promise.reject(new _errors.CraftAiBadRequestError('Bad Request, unable to create an agent with no or invalid configuration provided.'));
      }

      if (isUnvalidId(id)) {
        return Promise.reject(new _errors.CraftAiBadRequestError("Bad Request, unable to create an agent with invalid agent id. It must only contain characters in 'a-zA-Z0-9_-' and must be a string between 1 and ".concat(_constants.AGENT_ID_MAX_LENGTH, " characters.")));
      }

      return request({
        method: 'POST',
        path: '/agents',
        body: {
          id: id,
          configuration: configuration
        }
      }).then(function (_ref) {
        var body = _ref.body;
        debug("Agent '".concat(body.id, "' created."));
        return body;
      });
    },
    createAgentBulk: function createAgentBulk(agentsList) {
      checkBulkParameters(agentsList);
      return request({
        method: 'POST',
        path: '/bulk/agents',
        body: agentsList
      }).then(function (_ref2) {
        var body = _ref2.body;
        return body;
      });
    },
    getAgent: function getAgent(agentId) {
      if (!_constants.AGENT_ID_ALLOWED_REGEXP.test(agentId)) {
        return Promise.reject(new _errors.CraftAiBadRequestError('Bad Request, unable to get an agent with invalid agent id. It must only contain characters in \'a-zA-Z0-9_-\' and cannot be the empty string.'));
      }

      return request({
        method: 'GET',
        path: "/agents/".concat(agentId)
      }).then(function (_ref3) {
        var body = _ref3.body;
        return body;
      });
    },
    listAgents: function listAgents(agentId) {
      return request({
        method: 'GET',
        path: '/agents'
      }).then(function (_ref4) {
        var body = _ref4.body;
        return body.agentsList;
      });
    },
    deleteAgent: function deleteAgent(agentId) {
      if (_.isUndefined(agentId)) {
        return Promise.reject(new _errors.CraftAiBadRequestError('Bad Request, unable to delete an agent with no agentId provided.'));
      }

      return request({
        method: 'DELETE',
        path: "/agents/".concat(agentId)
      }).then(function (_ref5) {
        var body = _ref5.body;
        debug("Agent '".concat(agentId, "' deleted"));
        return body;
      });
    },
    deleteAgentBulk: function deleteAgentBulk(agentsList) {
      checkBulkParameters(agentsList);
      return request({
        method: 'DELETE',
        path: '/bulk/agents',
        body: agentsList
      }).then(function (_ref6) {
        var body = _ref6.body;
        return body;
      });
    },
    destroyAgent: function destroyAgent(agentId) {
      (0, _constants.deprecation)('client.destroyAgent', 'client.deleteAgent');
      return this.deleteAgent(agentId);
    },
    getAgentContext: function getAgentContext(agentId) {
      var t = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : undefined;

      if (_.isUndefined(agentId)) {
        return Promise.reject(new _errors.CraftAiBadRequestError('Bad Request, unable to get the agent context with no agentId provided.'));
      }

      var posixTimestamp;

      try {
        posixTimestamp = (0, _time["default"])(t).timestamp;
      } catch (err) {
        return Promise.reject(err);
      }

      return request({
        method: 'GET',
        path: "/agents/".concat(agentId, "/context/state"),
        query: {
          t: posixTimestamp
        }
      }).then(function (_ref7) {
        var body = _ref7.body;
        return body;
      });
    },
    addAgentContextOperations: function addAgentContextOperations(agentId, operations) {
      if (_.isUndefined(agentId)) {
        return Promise.reject(new _errors.CraftAiBadRequestError('Bad Request, unable to add agent context operations with no agentId provided.'));
      }

      if (!_.isArray(operations)) {
        // Only one given operation
        operations = [operations];
      }

      operations = _.compact(operations);

      if (!operations.length) {
        var message = "No operation to add to the agent ".concat(cfg.owner, "/").concat(cfg.project, "/").concat(agentId, " context.");
        debug(message);
        return Promise.resolve({
          message: message
        });
      }

      var totalNbOperationsAdded = 0;
      var sortedOperations = operations.map(function (_ref8) {
        var context = _ref8.context,
            timestamp = _ref8.timestamp;
        return {
          context: context,
          timestamp: (0, _time["default"])(timestamp).timestamp
        };
      }).sort(function (samp1, samp2) {
        return samp1.timestamp - samp2.timestamp;
      });
      return _.chunk(sortedOperations, cfg.operationsChunksSize).reduce(function (acc, chunk) {
        return acc.then(function () {
          return request({
            method: 'POST',
            path: "/agents/".concat(agentId, "/context"),
            body: chunk
          });
        }).then(function (response) {
          var nbOperationsAdded = response.body.nbOperationsAdded;
          totalNbOperationsAdded += nbOperationsAdded;
          return response;
        });
      }, Promise.resolve()).then(function () {
        var message = "Successfully added ".concat(totalNbOperationsAdded, " operation(s) to the agent ").concat(cfg.owner, "/").concat(cfg.project, "/").concat(agentId, " context.");
        debug(message);
        return {
          message: message,
          nbOperationsAdded: totalNbOperationsAdded
        };
      });
    },
    addAgentContextOperationsBulk: function addAgentContextOperationsBulk(agentsOperationsList) {
      var _this = this;

      checkBulkParameters(agentsOperationsList);
      agentsOperationsList.map(function (_ref9) {
        var id = _ref9.id,
            operations = _ref9.operations;

        if (areUnvalidOperations(operations)) {
          throw new _errors.CraftAiBadRequestError("Bad Request, unable to handle operations for agent ".concat(id, ". Operations should be provided within an array."));
        }
      });
      var chunkedData = [];
      var currentChunk = [];
      var currentChunkSize = 0;

      var _iterator = _createForOfIteratorHelper(agentsOperationsList),
          _step;

      try {
        for (_iterator.s(); !(_step = _iterator.n()).done;) {
          var agent = _step.value;

          if (agent.operations && _.isArray(agent.operations)) {
            if (currentChunkSize + agent.operations.length > cfg.operationsChunksSize && currentChunkSize.length) {
              chunkedData.push(currentChunk);
              currentChunkSize = 0;
              currentChunk = [];
            }

            if (agent.operations.length > cfg.operationsChunksSize) {
              chunkedData.push([agent]);
              currentChunkSize = 0;
            } else {
              currentChunkSize += agent.operations.length;
              currentChunk.push(agent);
            }
          }
        }
      } catch (err) {
        _iterator.e(err);
      } finally {
        _iterator.f();
      }

      if (currentChunk.length) {
        chunkedData.push(currentChunk);
      }

      return chunkedData.reduce(function (acc, chunk) {
        return acc.then(function (resultArray) {
          if (chunk.length > 1) {
            return request({
              method: 'POST',
              path: '/bulk/context',
              body: chunk
            }).then(function (_ref10) {
              var body = _ref10.body;
              resultArray.push(body);
              return resultArray;
            });
          } else {
            return _this.addAgentContextOperations(chunk[0].id, chunk[0].operations).then(function (_ref11) {
              var message = _ref11.message,
                  nbOperationsAdded = _ref11.nbOperationsAdded;
              var result = [{
                id: chunk[0].id,
                status: 201,
                message: message,
                nbOperationsAdded: nbOperationsAdded
              }];
              resultArray.push(result);
              return resultArray;
            });
          }
        });
      }, Promise.resolve([])).then(_.flattenDeep);
    },
    getAgentContextOperations: function getAgentContextOperations(agentId) {
      var _this2 = this;

      var start = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : undefined;
      var end = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : undefined;

      if (_.isUndefined(agentId)) {
        return Promise.reject(new _errors.CraftAiBadRequestError('Bad Request, unable to get agent context operations with no agentId provided.'));
      }

      var startTimestamp;

      if (start) {
        startTimestamp = (0, _time["default"])(start).timestamp;

        if (_.isUndefined(startTimestamp)) {
          return Promise.reject(new _errors.CraftAiBadRequestError('Bad Request, unable to get agent context operations with an invalid \'start\' timestamp provided.'));
        }
      }

      var endTimestamp;

      if (end) {
        endTimestamp = (0, _time["default"])(end).timestamp;

        if (_.isUndefined(endTimestamp)) {
          return Promise.reject(new _errors.CraftAiBadRequestError('Bad Request, unable to get agent context operations with an invalid \'end\' timestamp provided.'));
        }
      }

      var requestFollowingPages = function requestFollowingPages(_ref12) {
        var operations = _ref12.operations,
            nextPageUrl = _ref12.nextPageUrl;

        if (!nextPageUrl) {
          return Promise.resolve(operations);
        }

        return request({
          url: nextPageUrl
        }, _this2).then(function (_ref13) {
          var body = _ref13.body,
              nextPageUrl = _ref13.nextPageUrl;
          return requestFollowingPages({
            operations: operations.concat(body),
            nextPageUrl: nextPageUrl
          });
        });
      };

      return request({
        method: 'GET',
        path: "/agents/".concat(agentId, "/context"),
        query: {
          start: startTimestamp,
          end: endTimestamp
        }
      }).then(function (_ref14) {
        var body = _ref14.body,
            nextPageUrl = _ref14.nextPageUrl;
        return requestFollowingPages({
          operations: body,
          nextPageUrl: nextPageUrl
        });
      });
    },
    getAgentStateHistory: function getAgentStateHistory(agentId) {
      var start = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : undefined;
      var end = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : undefined;

      if (_.isUndefined(agentId)) {
        return Promise.reject(new _errors.CraftAiBadRequestError('Bad Request, unable to get agent state history with no agentId provided.'));
      }

      var startTimestamp;

      if (start) {
        startTimestamp = (0, _time["default"])(start).timestamp;

        if (_.isUndefined(startTimestamp)) {
          return Promise.reject(new _errors.CraftAiBadRequestError('Bad Request, unable to get agent state history with an invalid \'start\' timestamp provided.'));
        }
      }

      var endTimestamp;

      if (end) {
        endTimestamp = (0, _time["default"])(end).timestamp;

        if (_.isUndefined(endTimestamp)) {
          return Promise.reject(new _errors.CraftAiBadRequestError('Bad Request, unable to get agent state history with an invalid \'end\' timestamp provided.'));
        }
      }

      var requestFollowingPages = function requestFollowingPages(_ref15) {
        var stateHistory = _ref15.stateHistory,
            nextPageUrl = _ref15.nextPageUrl;

        if (!nextPageUrl) {
          return Promise.resolve(stateHistory);
        }

        return request({
          url: nextPageUrl
        }).then(function (_ref16) {
          var body = _ref16.body,
              nextPageUrl = _ref16.nextPageUrl;
          return requestFollowingPages({
            stateHistory: stateHistory.concat(body),
            nextPageUrl: nextPageUrl
          });
        });
      };

      return request({
        method: 'GET',
        path: "/agents/".concat(agentId, "/context/state/history"),
        query: {
          start: startTimestamp,
          end: endTimestamp
        }
      }).then(function (_ref17) {
        var body = _ref17.body,
            nextPageUrl = _ref17.nextPageUrl;
        return requestFollowingPages({
          stateHistory: body,
          nextPageUrl: nextPageUrl
        });
      });
    },
    getAgentInspectorUrl: function getAgentInspectorUrl(agentId) {
      var t = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : undefined;
      (0, _constants.deprecation)('client.getAgentInspectorUrl', 'client.getSharedAgentInspectorUrl');
      return this.getSharedAgentInspectorUrl(agentId, t);
    },
    getSharedAgentInspectorUrl: function getSharedAgentInspectorUrl(agentId) {
      var t = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : undefined;
      return request({
        method: 'GET',
        path: "/agents/".concat(agentId, "/shared")
      }).then(function (_ref18) {
        var body = _ref18.body;

        if (_.isUndefined(t)) {
          return body.shortUrl;
        } else {
          var posixTimestamp = (0, _time["default"])(t).timestamp;
          return "".concat(body.shortUrl, "?t=").concat(posixTimestamp);
        }
      });
    },
    deleteSharedAgentInspectorUrl: function deleteSharedAgentInspectorUrl(agentId) {
      return request({
        method: 'DELETE',
        path: "/agents/".concat(agentId, "/shared")
      }).then(function () {
        debug("Delete shared inspector link for agent '".concat(agentId, "'."));
      });
    },
    getAgentDecisionTree: function getAgentDecisionTree(agentId) {
      var t = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : undefined;
      var version = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : _constants.DEFAULT_DECISION_TREE_VERSION;

      if (_.isUndefined(agentId)) {
        return Promise.reject(new _errors.CraftAiBadRequestError('Bad Request, unable to retrieve an agent decision tree with no agentId provided.'));
      }

      var posixTimestamp = (0, _time["default"])(t).timestamp;

      if (_.isUndefined(posixTimestamp)) {
        return Promise.reject(new _errors.CraftAiBadRequestError('Bad Request, unable to retrieve an agent decision tree with an invalid timestamp provided.'));
      }

      var agentDecisionTreeRequest = function agentDecisionTreeRequest() {
        return request({
          method: 'GET',
          path: "/agents/".concat(agentId, "/decision/tree"),
          query: {
            t: posixTimestamp
          },
          headers: {
            'x-craft-ai-tree-version': version
          }
        }).then(function (_ref19) {
          var body = _ref19.body;
          return body;
        });
      };

      if (!cfg.decisionTreeRetrievalTimeout) {
        // Don't retry
        return agentDecisionTreeRequest();
      } else {
        var start = Date.now();
        return Promise.race([agentDecisionTreeRequest()["catch"](function (error) {
          var requestDuration = Date.now() - start;
          var expectedRetryDuration = requestDuration + 2000; // Let's add some margin

          var timeoutBeforeRetrying = cfg.decisionTreeRetrievalTimeout - requestDuration - expectedRetryDuration;

          if (error instanceof _errors.CraftAiLongRequestTimeOutError && timeoutBeforeRetrying > 0) {
            // First timeout, let's retry once near the end of the set timeout
            return resolveAfterTimeout(timeoutBeforeRetrying).then(function () {
              return agentDecisionTreeRequest();
            });
          } else {
            return Promise.reject(error);
          }
        }), resolveAfterTimeout(cfg.decisionTreeRetrievalTimeout).then(function () {
          throw new _errors.CraftAiLongRequestTimeOutError();
        })]);
      }
    },
    getAgentDecisionTreeBulk: function getAgentDecisionTreeBulk(agentsList) {
      checkBulkParameters(agentsList);
      return request({
        method: 'POST',
        path: '/bulk/decision_tree',
        body: agentsList
      }).then(function (_ref20) {
        var body = _ref20.body;
        return body;
      });
    },
    computeAgentDecision: function computeAgentDecision(agentId, t) {
      for (var _len = arguments.length, contexts = new Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
        contexts[_key - 2] = arguments[_key];
      }

      if (_.isUndefined(agentId)) {
        return Promise.reject(new _errors.CraftAiBadRequestError('Bad Request, unable to compute an agent decision with no agentId provided.'));
      }

      var posixTimestamp = (0, _time["default"])(t).timestamp;

      if (_.isUndefined(posixTimestamp)) {
        return Promise.reject(new _errors.CraftAiBadRequestError('Bad Request, unable to compute an agent decision with no or invalid timestamp provided.'));
      }

      if (_.isUndefined(contexts) || _.size(contexts) === 0 || _.size(contexts) === 1 && !contexts[0]) {
        return Promise.reject(new _errors.CraftAiBadRequestError('Bad Request, unable to compute an agent decision with no context provided.'));
      }

      return request({
        method: 'GET',
        path: "/agents/".concat(agentId, "/decision/tree"),
        query: {
          t: posixTimestamp
        }
      }).then(function (_ref21) {
        var body = _ref21.body;

        var decision = _interpreter.decide.apply(void 0, [body].concat(contexts));

        decision.timestamp = posixTimestamp;
        return decision;
      });
    },
    computeAgentBoostingDecision: function computeAgentBoostingDecision(agentName, fromTs, toTs, context) {
      if (_.isUndefined(agentName)) {
        return Promise.reject(new _errors.CraftAiBadRequestError('Bad Request, unable to compute an agent boosting decision with no agent name provided.'));
      }

      if (_.isUndefined(context)) {
        return Promise.reject(new _errors.CraftAiBadRequestError('Bad Request, unable to compute an agent boosting decision with no context provided.'));
      }

      return request({
        method: 'POST',
        path: "/agents/".concat(agentName, "/boosting/decision"),
        body: {
          timeWindow: [fromTs, toTs],
          context: context
        }
      }).then(function (_ref22) {
        var body = _ref22.body;
        return body;
      });
    },
    computeAgentBoostingDecisionBulk: function computeAgentBoostingDecisionBulk(agentsList) {
      checkBulkParameters(agentsList);
      return request({
        method: 'POST',
        path: '/bulk/boosting/decision',
        body: agentsList
      }).then(function (_ref23) {
        var body = _ref23.body;
        return body;
      });
    },
    // Generators methods
    createGenerator: function createGenerator(configuration, generatorName) {
      if (isUnvalidConfiguration(configuration)) {
        return Promise.reject(new _errors.CraftAiBadRequestError('Bad Request, unable to create a generator with no or invalid configuration provided.'));
      }

      if (isUnvalidId(generatorName)) {
        return Promise.reject(new _errors.CraftAiBadRequestError("Bad Request, unable to create a generator with invalid agent id. It must only contain characters in 'a-zA-Z0-9_-' and must be a string between 1 and ".concat(_constants.AGENT_ID_MAX_LENGTH, " characters.")));
      }

      if (isInvalidFilter(configuration.filter)) {
        return Promise.reject(new _errors.CraftAiBadRequestError("Bad Request, unable to create a generator with invalid filter. It must be a list of string containing characters in 'a-zA-Z0-9_-' and must be between 1 and ".concat(_constants.AGENT_ID_MAX_LENGTH, " characters.")));
      }

      return request({
        method: 'POST',
        path: '/generators',
        body: {
          id: generatorName,
          configuration: configuration
        }
      }).then(function (_ref24) {
        var body = _ref24.body;
        debug("Generator '".concat(body.id, "' created."));
        return body;
      });
    },
    createGeneratorBulk: function createGeneratorBulk(generatorsList) {
      checkBulkParameters(generatorsList);
      return request({
        method: 'POST',
        path: '/bulk/generators',
        body: generatorsList
      }).then(function (_ref25) {
        var body = _ref25.body;
        return body;
      });
    },
    deleteGenerator: function deleteGenerator(generatorName) {
      if (isUnvalidId(generatorName)) {
        return Promise.reject(new _errors.CraftAiBadRequestError("Bad Request, unable to delete a generator from an invalid generator id. It must only contain characters in 'a-zA-Z0-9_-' and must be a string between 1 and ".concat(_constants.AGENT_ID_MAX_LENGTH, " characters.")));
      }

      return request({
        method: 'DELETE',
        path: "/generators/".concat(generatorName)
      }).then(function (_ref26) {
        var body = _ref26.body;
        debug("Generator '".concat(generatorName, "' deleted"));
        return body;
      });
    },
    deleteGeneratorBulk: function deleteGeneratorBulk(generatorsList) {
      checkBulkParameters(generatorsList);
      return request({
        method: 'DELETE',
        path: '/bulk/generators',
        body: generatorsList
      }).then(function (_ref27) {
        var body = _ref27.body;
        return body;
      });
    },
    getGenerator: function getGenerator(generatorId) {
      if (isUnvalidId(generatorId)) {
        return Promise.reject(new _errors.CraftAiBadRequestError('Bad Request, unable to get an generator with invalid generator id. It must only contain characters in \'a-zA-Z0-9_-\' and cannot be the empty string.'));
      }

      return request({
        method: 'GET',
        path: "/generators/".concat(generatorId)
      }).then(function (_ref28) {
        var body = _ref28.body;
        return body;
      });
    },
    listGenerators: function listGenerators() {
      return request({
        method: 'GET',
        path: '/generators'
      }).then(function (_ref29) {
        var body = _ref29.body;
        return body.generatorsList;
      });
    },
    getGeneratorContextOperations: function getGeneratorContextOperations(generatorUri) {
      var _this3 = this;

      var start = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : undefined;
      var end = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : undefined;

      if (_.isUndefined(generatorUri)) {
        return Promise.reject(new _errors.CraftAiBadRequestError('Bad Request, unable to get generator context operations with no generatorUri provided.'));
      }

      var startTimestamp;

      if (start) {
        startTimestamp = (0, _time["default"])(start).timestamp;

        if (_.isUndefined(startTimestamp)) {
          return Promise.reject(new _errors.CraftAiBadRequestError('Bad Request, unable to get generator context operations with an invalid \'start\' timestamp provided.'));
        }
      }

      var endTimestamp;

      if (end) {
        endTimestamp = (0, _time["default"])(end).timestamp;

        if (_.isUndefined(endTimestamp)) {
          return Promise.reject(new _errors.CraftAiBadRequestError('Bad Request, unable to get generator context operations with an invalid \'end\' timestamp provided.'));
        }
      }

      var requestFollowingPages = function requestFollowingPages(_ref30) {
        var operations = _ref30.operations,
            nextPageUrl = _ref30.nextPageUrl;

        if (!nextPageUrl) {
          return Promise.resolve(operations);
        }

        return request({
          url: nextPageUrl
        }, _this3).then(function (_ref31) {
          var body = _ref31.body,
              nextPageUrl = _ref31.nextPageUrl;
          return requestFollowingPages({
            operations: operations.concat(body),
            nextPageUrl: nextPageUrl
          });
        });
      };

      return request({
        method: 'GET',
        path: "/generators/".concat(generatorUri, "/context"),
        query: {
          start: startTimestamp,
          end: endTimestamp
        }
      }).then(function (_ref32) {
        var body = _ref32.body,
            nextPageUrl = _ref32.nextPageUrl;
        return requestFollowingPages({
          operations: body,
          nextPageUrl: nextPageUrl
        });
      });
    },
    getGeneratorDecisionTree: function getGeneratorDecisionTree(generatorName) {
      var t = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : undefined;
      var version = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : _constants.DEFAULT_DECISION_TREE_VERSION;

      if (isUnvalidId(generatorName)) {
        return Promise.reject(new _errors.CraftAiBadRequestError("Bad Request, unable to get a decision tree from an invalid generator id. It must only contain characters in 'a-zA-Z0-9_-' and must be a string between 1 and ".concat(_constants.AGENT_ID_MAX_LENGTH, " characters.")));
      }

      var posixTimestamp = (0, _time["default"])(t).timestamp;

      if (_.isUndefined(posixTimestamp)) {
        return Promise.reject(new _errors.CraftAiBadRequestError('Bad Request, unable to retrieve an generator decision tree with an invalid timestamp provided.'));
      }

      var generatorDecisionTreeRequest = function generatorDecisionTreeRequest() {
        return request({
          method: 'GET',
          path: "/generators/".concat(generatorName, "/tree"),
          query: {
            t: posixTimestamp
          },
          headers: {
            'x-craft-ai-tree-version': version
          }
        }).then(function (_ref33) {
          var body = _ref33.body;
          return body;
        });
      };

      if (!cfg.decisionTreeRetrievalTimeout) {
        // Don't retry
        return generatorDecisionTreeRequest();
      } else {
        var start = Date.now();
        return Promise.race([generatorDecisionTreeRequest()["catch"](function (error) {
          var requestDuration = Date.now() - start;
          var expectedRetryDuration = requestDuration + 2000; // Let's add some margin

          var timeoutBeforeRetrying = cfg.decisionTreeRetrievalTimeout - requestDuration - expectedRetryDuration;

          if (error instanceof _errors.CraftAiLongRequestTimeOutError && timeoutBeforeRetrying > 0) {
            // First timeout, let's retry once near the end of the set timeout
            return resolveAfterTimeout(timeoutBeforeRetrying).then(function () {
              return generatorDecisionTreeRequest();
            });
          } else {
            return Promise.reject(error);
          }
        }), resolveAfterTimeout(cfg.decisionTreeRetrievalTimeout).then(function () {
          throw new _errors.CraftAiLongRequestTimeOutError();
        })]);
      }
    },
    getGeneratorDecisionTreeBulk: function getGeneratorDecisionTreeBulk(generatorsList) {
      checkBulkParameters(generatorsList);
      return request({
        method: 'POST',
        path: '/bulk/generators/tree',
        body: generatorsList
      }).then(function (_ref34) {
        var body = _ref34.body;
        return body;
      });
    },
    computeGeneratorDecision: function computeGeneratorDecision(generatorName, t) {
      for (var _len2 = arguments.length, contexts = new Array(_len2 > 2 ? _len2 - 2 : 0), _key2 = 2; _key2 < _len2; _key2++) {
        contexts[_key2 - 2] = arguments[_key2];
      }

      if (_.isUndefined(generatorName)) {
        return Promise.reject(new _errors.CraftAiBadRequestError('Bad Request, unable to compute an agent decision with no generatorName provided.'));
      }

      var posixTimestamp;

      try {
        posixTimestamp = (0, _time["default"])(t).timestamp;
      } catch (err) {
        return Promise.reject(err);
      }

      if (_.isUndefined(contexts) || _.size(contexts) === 0 || _.size(contexts) === 1 && !contexts[0]) {
        return Promise.reject(new _errors.CraftAiBadRequestError('Bad Request, unable to compute an agent decision with no context provided.'));
      }

      return request({
        method: 'GET',
        path: "/generators/".concat(generatorName, "/tree"),
        query: {
          t: posixTimestamp
        }
      }).then(function (_ref35) {
        var body = _ref35.body;

        var decision = _interpreter.decide.apply(void 0, [body].concat(contexts));

        decision.timestamp = posixTimestamp;
        return decision;
      });
    },
    computeGeneratorBoostingDecision: function computeGeneratorBoostingDecision(generatorName, fromTs, toTs, context) {
      if (_.isUndefined(generatorName)) {
        return Promise.reject(new _errors.CraftAiBadRequestError('Bad Request, unable to compute an generator boosting decision with no generatorName provided.'));
      }

      if (_.isUndefined(context)) {
        return Promise.reject(new _errors.CraftAiBadRequestError('Bad Request, unable to compute an generator boosting decision with no context provided.'));
      }

      return request({
        method: 'POST',
        path: "/generators/".concat(generatorName, "/boosting/decision"),
        body: {
          timeWindow: [fromTs, toTs],
          context: context
        }
      }).then(function (_ref36) {
        var body = _ref36.body;
        return body;
      });
    },
    computeGeneratorBoostingDecisionBulk: function computeGeneratorBoostingDecisionBulk(generatorsList) {
      checkBulkParameters(generatorsList);
      return request({
        method: 'POST',
        path: '/bulk/generators/boosting/decision',
        body: generatorsList
      }).then(function (_ref37) {
        var body = _ref37.body;
        return body;
      });
    },
    getSlidingWindowScoresBulk: function getSlidingWindowScoresBulk(scoreRequests) {
      checkBulkParameters(scoreRequests);
      return request({
        method: 'POST',
        path: '/bulk/generators/score/sliding',
        body: scoreRequests
      }).then(function (_ref38) {
        var body = _ref38.body;
        return body;
      });
    },
    getSingleWindowScoreBulk: function getSingleWindowScoreBulk(scoreRequests) {
      checkBulkParameters(scoreRequests);
      return request({
        method: 'POST',
        path: '/bulk/generators/score/single',
        body: scoreRequests
      }).then(function (_ref39) {
        var body = _ref39.body;
        return body;
      });
    }
  };
  return instance;
}