"use strict";

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.formatProperty = formatProperty;
exports.formatDecisionRules = formatDecisionRules;

var _time2 = _interopRequireDefault(require("./time"));

var _ = _interopRequireWildcard(require("./lodash"));

var _constants = require("./constants");

var _PROPERTY_FORMATTER, _OPERATORS$IN, _FORMATTER_FROM_DECIS;

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var DAYS = ['Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun'];
var MONTH = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'];
var PROPERTY_FORMATTER = (_PROPERTY_FORMATTER = {}, _defineProperty(_PROPERTY_FORMATTER, _constants.TYPE_ANY, function (value) {
  return value;
}), _defineProperty(_PROPERTY_FORMATTER, _constants.TYPES["boolean"], function (bool) {
  return bool.toString();
}), _defineProperty(_PROPERTY_FORMATTER, _constants.TYPES.continuous, function (number) {
  return number > 0.01 ? "".concat(Math.round(number * 100) / 100) : number.toExponential(2);
}), _defineProperty(_PROPERTY_FORMATTER, _constants.TYPES.time_of_day, function (time) {
  var _time = time instanceof _time2["default"] ? time.time_of_day : time;

  var hours = Math.floor(_time);

  var hoursStr = _.padStart(hours, 2, '0');

  var decMinutes = Math.round((_time - hours) * 60 * 100) / 100;
  var minutes = Math.floor(decMinutes);

  var minutesStr = _.padStart(minutes, 2, '0');

  var seconds = Math.round((decMinutes - minutes) * 60);

  var secondsStr = _.padStart(seconds, 2, '0');

  if (seconds > 0) {
    return "".concat(hoursStr, ":").concat(minutesStr, ":").concat(secondsStr);
  } else {
    return "".concat(hoursStr, ":").concat(minutesStr);
  }
}), _defineProperty(_PROPERTY_FORMATTER, _constants.TYPES.day_of_week, function (day) {
  var _day = day instanceof _time2["default"] ? day.day_of_week : day;

  return DAYS[_day];
}), _defineProperty(_PROPERTY_FORMATTER, _constants.TYPES.day_of_month, function (day) {
  var _day = day instanceof _time2["default"] ? day.day_of_month : day;

  return _.padStart(_day, 2, '0');
}), _defineProperty(_PROPERTY_FORMATTER, _constants.TYPES.month_of_year, function (month) {
  var _month = month instanceof _time2["default"] ? month.month_of_year : month;

  return MONTH[_month - 1];
}), _PROPERTY_FORMATTER);

function formatProperty(type) {
  var value = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : undefined;
  var formatter = PROPERTY_FORMATTER[type] || PROPERTY_FORMATTER[_constants.TYPE_ANY];

  var extendedFormatter = function extendedFormatter(value) {
    // A `null` value corresponds to a null/MVs branch
    if (_.isNull(value)) {
      return 'null';
    } // The empty object `{}` corresponds to an optional branch
    else if (_.isPlainObject(value) && _.isEmpty(value)) {
        return 'N/A';
      }

    return formatter(value);
  };

  if (!_.isUndefined(value)) {
    return extendedFormatter(value);
  }

  return extendedFormatter;
}

var FORMATTER_FROM_DECISION_RULE = (_FORMATTER_FROM_DECIS = {}, _defineProperty(_FORMATTER_FROM_DECIS, _constants.OPERATORS.IS, _defineProperty({}, _constants.TYPE_ANY, function (_ref) {
  var property = _ref.property,
      operand = _ref.operand,
      operandFormatter = _ref.operandFormatter;

  if (property) {
    return "'".concat(property, "' is ").concat(operandFormatter(operand));
  }

  return "is ".concat(operandFormatter(operand));
})), _defineProperty(_FORMATTER_FROM_DECIS, _constants.OPERATORS.IN, (_OPERATORS$IN = {}, _defineProperty(_OPERATORS$IN, _constants.TYPE_ANY, function (_ref2) {
  var property = _ref2.property,
      operand = _ref2.operand,
      operandFormatter = _ref2.operandFormatter;

  if (property) {
    return "'".concat(property, "' in [").concat(operandFormatter(operand[0]), ", ").concat(operandFormatter(operand[1]), "[");
  }

  return "[".concat(operandFormatter(operand[0]), ", ").concat(operandFormatter(operand[1]), "[");
}), _defineProperty(_OPERATORS$IN, _constants.TYPES.day_of_week, function (_ref3) {
  var property = _ref3.property,
      operand = _ref3.operand,
      operandFormatter = _ref3.operandFormatter;
  var day_from = Math.floor(operand[0]);
  var day_to = Math.floor(operand[1]); // If there is only one day in the interval

  if (day_to - day_from == 1 || day_from == 6 && day_to == 0) {
    if (property) {
      return "'".concat(property, "' is ").concat(operandFormatter(day_from));
    }

    return operandFormatter(day_from);
  } else {
    if (property) {
      return "'".concat(property, "' from ").concat(operandFormatter(day_from), " to ").concat(operandFormatter((7 + day_to - 1) % 7));
    }

    return "".concat(operandFormatter(day_from), " to ").concat(operandFormatter((7 + day_to - 1) % 7));
  }
}), _defineProperty(_OPERATORS$IN, _constants.TYPES.month_of_year, function (_ref4) {
  var property = _ref4.property,
      operand = _ref4.operand,
      operandFormatter = _ref4.operandFormatter;
  var month_from = Math.floor(operand[0]);
  var month_to = Math.floor(operand[1]);

  if (month_to - month_from == 1 || month_from == 12 && month_to == 1) {
    // One month in the interval
    if (property) {
      return "'".concat(property, "' is ").concat(operandFormatter(month_from));
    }

    return operandFormatter(month_from);
  } else if (month_to == 1) {
    // (Excluded) upper bound is january
    if (property) {
      return "'".concat(property, "' from ").concat(operandFormatter(month_from), " to ").concat(operandFormatter(12));
    }

    return "".concat(operandFormatter(month_from), " to ").concat(operandFormatter(12));
  } else {
    if (property) {
      return "'".concat(property, "' from ").concat(operandFormatter(month_from), " to ").concat(operandFormatter(month_to - 1));
    }

    return "".concat(operandFormatter(month_from), " to ").concat(operandFormatter(month_to - 1));
  }
}), _OPERATORS$IN)), _defineProperty(_FORMATTER_FROM_DECIS, _constants.OPERATORS.GTE, _defineProperty({}, _constants.TYPE_ANY, function (_ref5) {
  var property = _ref5.property,
      operand = _ref5.operand,
      operandFormatter = _ref5.operandFormatter;

  if (property) {
    return "'".concat(property, "' >= ").concat(operandFormatter(operand));
  }

  return ">= ".concat(operandFormatter(operand));
})), _defineProperty(_FORMATTER_FROM_DECIS, _constants.OPERATORS.LT, _defineProperty({}, _constants.TYPE_ANY, function (_ref6) {
  var property = _ref6.property,
      operand = _ref6.operand,
      operandFormatter = _ref6.operandFormatter;

  if (property) {
    return "'".concat(property, "' < ").concat(operandFormatter(operand));
  }

  return "< ".concat(operandFormatter(operand));
})), _FORMATTER_FROM_DECIS);

function formatDecisionRules(decisionRules) {
  return decisionRules.map(function (_ref7) {
    var property = _ref7.property,
        type = _ref7.type,
        operand = _ref7.operand,
        operator = _ref7.operator;
    var operatorFormatters = FORMATTER_FROM_DECISION_RULE[operator];

    if (!operatorFormatters) {
      throw new Error("Unable to format the given decision rule: unknown operator '".concat(operator, "'."));
    }

    var formatter = operatorFormatters[type] || operatorFormatters[_constants.TYPE_ANY];
    var operandFormatter = formatProperty(type || _constants.TYPE_ANY);
    return formatter({
      property: property,
      type: type,
      operator: operator,
      operandFormatter: operandFormatter,
      operand: operand
    });
  }).join(' and ');
}