"use strict";

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.decide = decide;
exports.getDecisionRulesProperties = getDecisionRulesProperties;
exports.decideFromContextsArray = decideFromContextsArray;
Object.defineProperty(exports, "reduceDecisionRules", {
  enumerable: true,
  get: function get() {
    return _reducer.reduceDecisionRules;
  }
});
Object.defineProperty(exports, "distribution", {
  enumerable: true,
  get: function get() {
    return _interpreter_v2.distribution;
  }
});
Object.defineProperty(exports, "formatDecisionRules", {
  enumerable: true,
  get: function get() {
    return _formatter.formatDecisionRules;
  }
});
Object.defineProperty(exports, "formatProperty", {
  enumerable: true,
  get: function get() {
    return _formatter.formatProperty;
  }
});

var _context = _interopRequireDefault(require("./context"));

var _interpreter_v = require("./interpreter_v1");

var _parse4 = _interopRequireDefault(require("./parse"));

var _reducer = require("./reducer");

var _semver = _interopRequireDefault(require("semver"));

var _ = _interopRequireWildcard(require("./lodash"));

var _errors = require("./errors");

var _interpreter_v2 = require("./interpreter_v2");

var _formatter = require("./formatter");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && Symbol.iterator in Object(iter)) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

var DECISION_FORMAT_VERSION = '1.1.0';

function decide(tree) {
  var _parse = (0, _parse4["default"])(tree),
      _version = _parse._version,
      configuration = _parse.configuration,
      trees = _parse.trees;

  for (var _len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    args[_key - 1] = arguments[_key];
  }

  var ctx = configuration ? _context["default"].apply(void 0, [configuration].concat(args)) : _.extend.apply(_, [{}].concat(args));
  return _decide(configuration, trees, ctx, _version);
}

function _decide(configuration, trees, ctx, _version) {
  if (_semver["default"].satisfies(_version, '>=1.0.0 <2.0.0')) {
    return (0, _interpreter_v.decide)(configuration, trees, ctx);
  }

  if (_semver["default"].satisfies(_version, '>=2.0.0 <3.0.0')) {
    return (0, _interpreter_v2.decide)(configuration, trees, ctx);
  }

  throw new _errors.CraftAiDecisionError("Invalid decision tree format, \"".concat(_version, "\" is not a valid version."));
}

function reduceNodes(tree, fn, initialAccValue) {
  var nodes = [];
  nodes.push(tree);

  var recursiveNext = function recursiveNext(acc) {
    if (nodes.length == 0) {
      // No more nodes
      return acc;
    }

    var node = nodes.pop();

    if (node.children) {
      nodes = node.children.concat(nodes);
    }

    var updatedAcc = fn(acc, node);
    return recursiveNext(updatedAcc);
  };

  return recursiveNext(initialAccValue);
}

function getDecisionRulesProperties(tree) {
  var _parse2 = (0, _parse4["default"])(tree),
      configuration = _parse2.configuration,
      trees = _parse2.trees;

  var reducedTrees = Object.values(trees).reduce(function (properties, tree) {
    return reduceNodes(tree, function (properties, node) {
      if (node.children) {
        // Skip leaves
        return properties.concat(node.children[0].decision_rule.property);
      }

      return properties;
    }, properties);
  }, []);
  return _.uniq(reducedTrees).map(function (property) {
    return _.extend(configuration.context[property], {
      property: property
    });
  });
}

function decideFromContextsArray(tree, contexts) {
  var _parse3 = (0, _parse4["default"])(tree),
      _version = _parse3._version,
      configuration = _parse3.configuration,
      trees = _parse3.trees;

  return _.map(contexts, function (contextsItem) {
    var ctx;

    if (_.isArray(contextsItem)) {
      ctx = _context["default"].apply(void 0, [configuration].concat(_toConsumableArray(contextsItem)));
    } else {
      ctx = (0, _context["default"])(configuration, contextsItem);
    }

    try {
      return _decide(configuration, trees, ctx, _version);
    } catch (error) {
      if (error instanceof _errors.CraftAiNullDecisionError) {
        var message = error.message,
            metadata = error.metadata;
        return {
          _version: DECISION_FORMAT_VERSION,
          context: ctx,
          error: {
            message: message,
            metadata: metadata
          }
        };
      } else {
        throw error;
      }
    }
  });
}