"use strict";

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.decide = decide;
Object.defineProperty(exports, "reduceDecisionRules", {
  enumerable: true,
  get: function get() {
    return _reducer.reduceDecisionRules;
  }
});
Object.defineProperty(exports, "formatDecisionRules", {
  enumerable: true,
  get: function get() {
    return _formatter.formatDecisionRules;
  }
});
Object.defineProperty(exports, "formatProperty", {
  enumerable: true,
  get: function get() {
    return _formatter.formatProperty;
  }
});

var _reducer = require("./reducer");

var _time = require("./time");

var _ = _interopRequireWildcard(require("./lodash"));

var _errors = require("./errors");

var _formatter = require("./formatter");

var _timezones = _interopRequireWildcard(require("./timezones"));

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && Symbol.iterator in Object(iter)) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var DECISION_FORMAT_VERSION = '1.1.0';
var OPERATORS = {
  'is': function is(context, value) {
    return context === value;
  },
  '>=': function _(context, value) {
    return context * 1 >= value;
  },
  '<': function _(context, value) {
    return context * 1 < value;
  },
  '[in[': function _in(context, value) {
    var context_val = context * 1;
    var from = value[0];
    var to = value[1]; //the interval is not looping

    if (from < to) {
      return context_val >= from && context_val < to;
    } //the interval IS looping
    else {
        return context_val >= from || context_val < to;
      }
  }
};
var VALUE_VALIDATOR = {
  continuous: function continuous(value) {
    return _.isFinite(value);
  },
  "enum": function _enum(value) {
    return _.isString(value);
  },
  timezone: function timezone(value) {
    return (0, _timezones["default"])(value);
  },
  time_of_day: function time_of_day(value) {
    return _.isFinite(value) && value >= 0 && value < 24;
  },
  day_of_week: function day_of_week(value) {
    return _.isInteger(value) && value >= 0 && value <= 6;
  },
  day_of_month: function day_of_month(value) {
    return _.isInteger(value) && value >= 1 && value <= 31;
  },
  month_of_year: function month_of_year(value) {
    return _.isInteger(value) && value >= 1 && value <= 12;
  }
};

function decideRecursion(node, context) {
  // Leaf
  if (!(node.children && node.children.length)) {
    if (node.predicted_value == null) {
      return {
        predicted_value: undefined,
        confidence: undefined,
        decision_rules: [],
        error: {
          name: 'CraftAiNullDecisionError',
          message: 'Unable to take decision: the decision tree has no valid predicted value for the given context.'
        }
      };
    }

    var leafNode = {
      predicted_value: node.predicted_value,
      confidence: node.confidence || 0,
      decision_rules: []
    };

    if (!_.isUndefined(node.standard_deviation)) {
      leafNode.standard_deviation = node.standard_deviation;
    }

    return leafNode;
  } // Regular node


  var matchingChild = _.find(node.children, function (child) {
    var decision_rule = child.decision_rule;
    var property = decision_rule.property;

    if (_.isUndefined(context[property])) {
      // Should not happen
      return {
        predicted_value: undefined,
        confidence: undefined,
        error: {
          name: 'CraftAiUnknownError',
          message: "Unable to take decision: property '".concat(property, "' is missing from the given context.")
        }
      };
    }

    return OPERATORS[decision_rule.operator](context[property], decision_rule.operand);
  }); // matching child property error


  if (matchingChild && matchingChild.error) {
    return matchingChild;
  }

  if (_.isUndefined(matchingChild)) {
    // Should only happens when an unexpected value for an enum is encountered
    var operandList = _.uniq(_.map(_.values(node.children), function (child) {
      return child.decision_rule.operand;
    }));

    var property = _.head(node.children).decision_rule.property;

    return {
      predicted_value: undefined,
      confidence: undefined,
      decision_rules: [],
      error: {
        name: 'CraftAiNullDecisionError',
        message: "Unable to take decision: value '".concat(context[property], "' for property '").concat(property, "' doesn't validate any of the decision rules."),
        metadata: {
          property: property,
          value: context[property],
          expected_values: operandList
        }
      }
    };
  } // matching child found: recurse !


  var result = decideRecursion(matchingChild, context);

  var finalResult = _.extend(result, {
    decision_rules: [matchingChild.decision_rule].concat(result.decision_rules)
  });

  return finalResult;
}

function checkContext(configuration) {
  // Extract the required properties (i.e. those that are not the output)
  var expectedProperties = _.difference(_.keys(configuration.context), configuration.output); // Build a context validator


  var validators = _.map(expectedProperties, function (property) {
    var otherValidator = function otherValidator() {
      console.warn("WARNING: \"".concat(configuration.context[property].type, "\" is not a supported type. Please refer to the documention to see what type you can use"));
      return true;
    };

    return {
      property: property,
      type: configuration.context[property].type,
      validator: VALUE_VALIDATOR[configuration.context[property].type] || otherValidator
    };
  });

  return function (context) {
    var _$reduce = _.reduce(validators, function (_ref, _ref2) {
      var badProperties = _ref.badProperties,
          missingProperties = _ref.missingProperties;
      var property = _ref2.property,
          type = _ref2.type,
          validator = _ref2.validator;
      var value = context[property];

      if (value === undefined) {
        missingProperties.push(property);
      } else if (!validator(value)) {
        badProperties.push({
          property: property,
          type: type,
          value: value
        });
      }

      return {
        badProperties: badProperties,
        missingProperties: missingProperties
      };
    }, {
      badProperties: [],
      missingProperties: []
    }),
        badProperties = _$reduce.badProperties,
        missingProperties = _$reduce.missingProperties;

    if (missingProperties.length || badProperties.length) {
      var messages = _.concat(_.map(missingProperties, function (property) {
        return "expected property '".concat(property, "' is not defined");
      }), _.map(badProperties, function (_ref3) {
        var property = _ref3.property,
            type = _ref3.type,
            value = _ref3.value;
        return "'".concat(value, "' is not a valid value for property '").concat(property, "' of type '").concat(type, "'");
      }));

      throw new _errors.CraftAiDecisionError({
        message: "Unable to take decision, the given context is not valid: ".concat(messages.join(', '), "."),
        metadata: _.assign({}, missingProperties.length && {
          missingProperties: missingProperties
        }, badProperties.length && {
          badProperties: badProperties
        })
      });
    }
  };
}

function decide(configuration, trees, context) {
  checkContext(configuration)(context); // Convert timezones as integers to the standard +/-hh:mm format
  // This should only happen when no Time() object is passed to the interpreter

  var timezoneProperty = (0, _timezones.getTimezoneKey)(configuration.context);
  var decide_context = timezoneProperty == null ? context : Object.assign({}, context, _defineProperty({}, timezoneProperty, (0, _time.tzFromOffset)(context[timezoneProperty])));
  return {
    _version: DECISION_FORMAT_VERSION,
    context: context,
    output: _.assign.apply(_, _toConsumableArray(_.map(configuration.output, function (output) {
      var root = trees[output];

      if (!(root.children && root.children.length)) {
        if (root.predicted_value === undefined) {
          throw new _errors.CraftAiNullDecisionError({
            message: 'Unable to take decision: the decision tree is not based on any context operations.'
          });
        }
      }

      var decision = decideRecursion(trees[output], decide_context);

      if (decision.error) {
        switch (decision.error.name) {
          case 'CraftAiNullDecisionError':
            throw new _errors.CraftAiNullDecisionError({
              message: decision.error.message,
              metadata: _.extend(decision.error.metadata, {
                decision_rules: decision.decision_rules
              })
            });

          default:
            throw new _errors.CraftAiUnknownError({
              message: decision.error.message
            });
        }
      }

      return _defineProperty({}, output, decision);
    })))
  };
}