"use strict";

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.distribution = distribution;
exports.computeMeanValues = computeMeanValues;
exports.computeMeanDistributions = computeMeanDistributions;
exports.decide = decide;
Object.defineProperty(exports, "reduceDecisionRules", {
  enumerable: true,
  get: function get() {
    return _reducer.reduceDecisionRules;
  }
});
Object.defineProperty(exports, "formatDecisionRules", {
  enumerable: true,
  get: function get() {
    return _formatter.formatDecisionRules;
  }
});
Object.defineProperty(exports, "formatProperty", {
  enumerable: true,
  get: function get() {
    return _formatter.formatProperty;
  }
});

var _reducer = require("./reducer");

var _time = require("./time");

var _2 = _interopRequireWildcard(require("./lodash"));

var _errors = require("./errors");

var _formatter = require("./formatter");

var _timezones = _interopRequireWildcard(require("./timezones"));

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && Symbol.iterator in Object(iter)) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return; var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

var DECISION_FORMAT_VERSION = '2.0.0';
var OPERATORS = {
  'is': function is(context, value) {
    if (_2.isObject(context) && _2.isObject(value)) {
      return _2.isEmpty(context) && _2.isEmpty(value);
    } else {
      return context === value;
    }
  },
  '>=': function _(context, value) {
    return !_2.isNull(context) && context * 1 >= value;
  },
  '<': function _(context, value) {
    return !_2.isNull(context) && context * 1 < value;
  },
  '[in[': function _in(context, value) {
    var context_val = context * 1;
    var from = value[0];
    var to = value[1]; //the interval is not looping

    if (from < to) {
      return !_2.isNull(context) && context_val >= from && context_val < to;
    } //the interval IS looping
    else {
        return !_2.isNull(context) && (context_val >= from || context_val < to);
      }
  },
  'in': function _in(context, value) {
    return value.indexOf(context) > -1;
  }
};
var VALUE_VALIDATOR = {
  continuous: function continuous(value) {
    return _2.isFinite(value);
  },
  "enum": function _enum(value) {
    return _2.isString(value);
  },
  "boolean": function boolean(value) {
    return _2.isBoolean(value);
  },
  timezone: function timezone(value) {
    return (0, _timezones["default"])(value);
  },
  time_of_day: function time_of_day(value) {
    return _2.isFinite(value) && value >= 0 && value < 24;
  },
  day_of_week: function day_of_week(value) {
    return _2.isInteger(value) && value >= 0 && value <= 6;
  },
  day_of_month: function day_of_month(value) {
    return _2.isInteger(value) && value >= 1 && value <= 31;
  },
  month_of_year: function month_of_year(value) {
    return _2.isInteger(value) && value >= 1 && value <= 12;
  }
};

function decideRecursion(node, context, configuration, outputType, outputValues) {
  var path = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : ['0'];

  // Leaf
  if (!(node.children && node.children.length)) {
    var prediction = node.prediction;

    if (prediction.value == null) {
      return {
        predicted_value: undefined,
        confidence: undefined,
        decision_rules: [],
        error: {
          name: 'CraftAiNullDecisionError',
          message: 'Unable to take decision: the decision tree has no valid predicted value for the given context.'
        }
      };
    }

    var leafNode = {
      predicted_value: prediction.value,
      confidence: prediction.confidence || 0,
      decision_rules: [],
      nb_samples: prediction.nb_samples,
      decision_path: path.join('-')
    };

    if (!_2.isUndefined(prediction.distribution.standard_deviation)) {
      leafNode.standard_deviation = prediction.distribution.standard_deviation;
      var min_value = prediction.distribution.min;
      var max_value = prediction.distribution.max;

      if (!_2.isUndefined(min_value)) {
        leafNode.min = min_value;
      }

      if (!_2.isUndefined(max_value)) {
        leafNode.max = max_value;
      }
    } else {
      leafNode.distribution = prediction.distribution;
    }

    return leafNode;
  } // Regular node


  var matchingChildIndex = node.children.findIndex(function (child) {
    var decision_rule = child.decision_rule;
    var property = decision_rule.property;
    return OPERATORS[decision_rule.operator](context[property], decision_rule.operand);
  });
  var matchingChild = node.children[matchingChildIndex]; // matching child property error

  if (matchingChild && matchingChild.error) {
    return matchingChild;
  }

  if (_2.isUndefined(matchingChild)) {
    var _distribution = distribution(node),
        value = _distribution.value,
        standard_deviation = _distribution.standard_deviation,
        size = _distribution.size;

    var _finalResult = {}; // If it is a classification problem we return the class with the highest
    // probability. Otherwise, if the current output type is continuous/periodic
    // then the returned value corresponds to the subtree weighted output values.

    if (outputType === 'enum' || outputType === 'boolean') {
      // Compute the argmax function on the returned distribution:
      var argmax = value.map(function (x, i) {
        return [x, i];
      }).reduce(function (r, a) {
        return a[0] > r[0] ? a : r;
      })[1];
      var predicted_value = outputValues[argmax];
      _finalResult = {
        predicted_value: predicted_value,
        distribution: value
      };
    } else {
      _finalResult = {
        predicted_value: value,
        standard_deviation: standard_deviation
      };
    }

    return _2.extend(_finalResult, {
      confidence: null,
      decision_rules: [],
      nb_samples: size,
      decision_path: path.join('-')
    });
  } // Add the matching child index to the current path:


  path.push(matchingChildIndex); // matching child found: recurse !

  var result = decideRecursion(matchingChild, context, configuration, outputType, outputValues, path);

  var finalResult = _2.extend(result, {
    decision_rules: [matchingChild.decision_rule].concat(result.decision_rules)
  });

  return finalResult;
}

function checkContext(configuration) {
  // Extract the required properties (i.e. those that are not the output)
  var expectedProperties = _2.difference(_2.keys(configuration.context), configuration.output); // Build a context validator


  var validators = _2.map(expectedProperties, function (property) {
    var otherValidator = function otherValidator() {
      console.warn("WARNING: \"".concat(configuration.context[property].type, "\" is not a supported type. Please refer to the documentation to see what type you can use"));
      return true;
    };

    return {
      property: property,
      type: configuration.context[property].type,
      validator: VALUE_VALIDATOR[configuration.context[property].type] || otherValidator
    };
  });

  return function (context) {
    var _$reduce = _2.reduce(validators, function (_ref, _ref2) {
      var badProperties = _ref.badProperties,
          missingProperties = _ref.missingProperties;
      var property = _ref2.property,
          type = _ref2.type,
          validator = _ref2.validator;
      var value = context[property];

      var isOptional = _2.isPlainObject(value) && _2.isEmpty(value);

      if (value === undefined) {
        missingProperties.push(property);
      } else if (!validator(value) && !_2.isNull(value) && !isOptional) {
        badProperties.push({
          property: property,
          type: type,
          value: value
        });
      }

      return {
        badProperties: badProperties,
        missingProperties: missingProperties
      };
    }, {
      badProperties: [],
      missingProperties: []
    }),
        badProperties = _$reduce.badProperties,
        missingProperties = _$reduce.missingProperties;

    if (missingProperties.length || badProperties.length) {
      var messages = _2.concat(_2.map(missingProperties, function (property) {
        return "expected property '".concat(property, "' is not defined");
      }), _2.map(badProperties, function (_ref3) {
        var property = _ref3.property,
            type = _ref3.type,
            value = _ref3.value;
        return "'".concat(value, "' is not a valid value for property '").concat(property, "' of type '").concat(type, "'");
      }));

      throw new _errors.CraftAiDecisionError({
        message: "Unable to take decision, the given context is not valid: ".concat(messages.join(', '), "."),
        metadata: _2.assign({}, missingProperties.length && {
          missingProperties: missingProperties
        }, badProperties.length && {
          badProperties: badProperties
        })
      });
    }
  };
}

function distribution(node) {
  if (!(node.children && node.children.length)) {
    // If the distribution attribute is an array it means that it is
    // a classification problem. We therefore compute the distribution of
    // the classes in this leaf and return the branch size.
    if (_2.isArray(node.prediction.distribution)) {
      return {
        value: node.prediction.distribution,
        size: node.prediction.nb_samples
      };
    } // Otherwise it is a regression problem, and we return the mean value
    // of the leaf, the standard_deviation and the branch size.


    return {
      value: node.prediction.value,
      standard_deviation: node.prediction.distribution.standard_deviation,
      size: node.prediction.nb_samples,
      min: node.prediction.distribution.min,
      max: node.prediction.distribution.max
    };
  } // If it is not a leaf, we recurse into the children and store the distributions
  // and sizes of each child branch.


  var _$map$reduce = _2.map(node.children, function (child) {
    return distribution(child);
  }).reduce(function (acc, _ref4) {
    var value = _ref4.value,
        standard_deviation = _ref4.standard_deviation,
        size = _ref4.size,
        min = _ref4.min,
        max = _ref4.max;
    acc.values.push(value);
    acc.sizes.push(size);

    if (!_2.isUndefined(standard_deviation)) {
      acc.stds.push(standard_deviation);
      acc.mins.push(min);
      acc.maxs.push(max);
    }

    return acc;
  }, {
    values: [],
    stds: [],
    sizes: [],
    mins: [],
    maxs: []
  }),
      values = _$map$reduce.values,
      stds = _$map$reduce.stds,
      sizes = _$map$reduce.sizes,
      mins = _$map$reduce.mins,
      maxs = _$map$reduce.maxs;

  if (_2.isArray(values[0])) {
    return computeMeanDistributions(values, sizes);
  }

  return computeMeanValues(values, sizes, stds, mins, maxs);
}

function computeMeanValues(values, sizes, stds, mins, maxs) {
  // Compute the weighted mean of the given array of values.
  // Example, for values = [ 4, 3, 6 ], sizes = [1, 2, 1]
  // This function computes (4*1 + 3*2 + 1*6) / (1+2+1) = 16/4 = 4
  // If no standard deviation array is given, use classical weighted mean formula:
  if (_2.isUndefined(stds)) {
    var totalSize = _2.sum(sizes);

    var newMean = _2.zip(values, sizes).map(function (_ref5) {
      var _ref6 = _slicedToArray(_ref5, 2),
          mean = _ref6[0],
          size = _ref6[1];

      return mean * (1.0 * size) / (1.0 * totalSize);
    }).reduce(_2.add);

    return {
      value: newMean,
      size: totalSize
    };
  } // Otherwise, to compute the weighted standard deviation the following formula is used:
  // https://math.stackexchange.com/questions/2238086/calculate-variance-of-a-subset


  var _$zip$map$reduce = _2.zip(values, stds, sizes, mins, maxs).map(function (_ref7) {
    var _ref8 = _slicedToArray(_ref7, 5),
        mean = _ref8[0],
        std = _ref8[1],
        size = _ref8[2],
        min = _ref8[3],
        max = _ref8[4];

    return {
      mean: mean,
      variance: std * std,
      size: size,
      min: min,
      max: max
    };
  }).reduce(function (acc, _ref9) {
    var mean = _ref9.mean,
        variance = _ref9.variance,
        size = _ref9.size,
        min = _ref9.min,
        max = _ref9.max;

    if (_2.isUndefined(acc.mean)) {
      return {
        mean: mean,
        variance: variance,
        size: size,
        min: min,
        max: max
      };
    }

    var totalSize = 1.0 * (acc.size + size);

    if (!totalSize > 0.0) {
      return {
        mean: acc.mean,
        variance: acc.variance,
        size: acc.size
      };
    }

    var newVariance = 1.0 / (totalSize - 1) * ((acc.size - 1) * acc.variance + (size - 1) * variance + acc.size * size / totalSize * (acc.mean - mean) * (acc.mean - mean));
    var newMean = 1.0 / totalSize * (acc.size * acc.mean + size * mean);
    var newMin = min < acc.min ? min : acc.min;
    var newMax = max > acc.max ? max : acc.max;
    return {
      mean: newMean,
      variance: newVariance,
      size: totalSize,
      min: newMin,
      max: newMax
    };
  }, {
    mean: undefined,
    variance: undefined,
    size: undefined,
    min: undefined,
    max: undefined
  }),
      mean = _$zip$map$reduce.mean,
      variance = _$zip$map$reduce.variance,
      size = _$zip$map$reduce.size,
      min = _$zip$map$reduce.min,
      max = _$zip$map$reduce.max;

  return {
    value: mean,
    standard_deviation: Math.sqrt(variance),
    size: size,
    min: min,
    max: max
  };
}

function computeMeanDistributions(values, sizes) {
  // Compute the weighted mean of the given array of distributions (array of probabilities).
  // Example, for values = [[ 4, 3, 6 ], [1, 2, 3], [3, 4, 5]], sizes = [1, 2, 1]
  // This function computes ([ 4, 3, 6]*1 + [1, 2, 3]*2 + [3, 4, 5]*6) / (1+2+1) = ...
  var totalSize = _2.sum(sizes);

  var multiplyByBranchRatio = _2.zip(values, sizes).map(function (zipped) {
    return _2.map(zipped[0], function (val) {
      return val * zipped[1] / totalSize;
    });
  });

  var sumArrays = _sumArrays(multiplyByBranchRatio);

  return {
    value: sumArrays,
    size: totalSize
  };
}

function _sumArrays(arrays) {
  return arrays.reduce(function (acc_sum, array) {
    return _2.map(array, function (val, i) {
      return (acc_sum[i] || 0.) + val;
    });
  }, new Array(arrays[0].length));
}

function decide(configuration, trees, context) {
  checkContext(configuration)(context); // Convert timezones as integers to the standard +/-hh:mm format
  // This should only happen when no Time() object is passed to the interpreter

  var timezoneProperty = (0, _timezones.getTimezoneKey)(configuration.context);
  var decide_context = timezoneProperty == null ? context : Object.assign({}, context, _defineProperty({}, timezoneProperty, (0, _time.tzFromOffset)(context[timezoneProperty])));
  return {
    _version: DECISION_FORMAT_VERSION,
    context: context,
    output: _2.assign.apply(_2, _toConsumableArray(_2.map(configuration.output, function (output) {
      var root = trees[output];

      if (!(root.children && root.children.length)) {
        var prediction = root.prediction;

        if (prediction.value == null) {
          throw new _errors.CraftAiNullDecisionError({
            message: 'Unable to take decision: the decision tree is not based on any context operations.'
          });
        }
      }

      var outputType = configuration.context[output].type;
      var decision = decideRecursion(trees[output], decide_context, configuration, outputType, trees[output].output_values);

      if (decision.error) {
        switch (decision.error.name) {
          case 'CraftAiNullDecisionError':
            throw new _errors.CraftAiNullDecisionError({
              message: decision.error.message,
              metadata: _2.extend(decision.error.metadata, {
                decision_rules: decision.decision_rules
              })
            });

          default:
            throw new _errors.CraftAiUnknownError({
              message: decision.error.message
            });
        }
      }

      return _defineProperty({}, output, decision);
    })))
  };
}