"use strict";

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = createRequest;

var _debug = _interopRequireDefault(require("debug"));

var _constants = require("./constants");

var _package = require("../package.json");

var _ = _interopRequireWildcard(require("./lodash"));

var _errors = require("./errors");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return; var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

var fetch = !_constants.IN_BROWSER && typeof fetch === 'undefined' ? require('node-fetch') : window.fetch;
var debug = (0, _debug["default"])('craft-ai:client');
var USER_AGENT = "craft-ai-client-js/".concat(_package.version, " [").concat(_constants.IN_BROWSER ? navigator.userAgent : "Node.js ".concat(process.version), "]");
debug("Client user agent set to '".concat(USER_AGENT, "'"));

function parseBody(req, resBody) {
  var resBodyUtf8;

  try {
    resBodyUtf8 = resBody.toString('utf-8');
  } catch (err) {
    debug("Invalid response format from ".concat(req.method, " ").concat(req.path, ": ").concat(resBody), err);
    throw new _errors.CraftAiInternalError('Internal Error, the craft ai server responded in an invalid format.', {
      request: req
    });
  }

  var resBodyJson;

  try {
    if (resBodyUtf8.length > 0) {
      resBodyJson = JSON.parse(resBodyUtf8);
    } else {
      resBodyJson = {};
    }
  } catch (err) {
    debug("Invalid json response from ".concat(req.method, " ").concat(req.path, ": ").concat(resBody), err);
    throw new _errors.CraftAiInternalError('Internal Error, the craft ai server responded an invalid json document.', {
      more: resBodyUtf8,
      request: req
    });
  }

  return resBodyJson;
}

function parseBulk(req, res, resBody) {
  if (_.isArray(JSON.parse(resBody))) {
    return {
      body: parseBody(req, resBody)
    };
  } else {
    throw new _errors.CraftAiBadRequestError({
      message: parseBody(req, resBody).message,
      request: req
    });
  }
}

function parseResponse(req, res, resBody) {
  switch (res.status) {
    case 200:
    case 201:
    case 204:
      return {
        body: parseBody(req, resBody),
        nextPageUrl: res.headers.get('x-craft-ai-next-page-url')
      };

    case 202:
      throw new _errors.CraftAiLongRequestTimeOutError({
        message: parseBody(req, resBody).message,
        request: req
      });

    case 207:
      return parseBulk(req, res, resBody);

    case 401:
    case 403:
      throw new _errors.CraftAiCredentialsError({
        message: parseBody(req, resBody).message,
        request: req
      });

    case 400:
    case 404:
      return parseBulk(req, res, resBody);

    case 413:
      throw new _errors.CraftAiBadRequestError({
        message: 'Given payload is too large',
        request: req
      });

    case 500:
      throw new _errors.CraftAiInternalError(parseBody(req, resBody).message, {
        request: req
      });

    case 504:
      throw new _errors.CraftAiInternalError({
        message: 'Response has timed out',
        request: req,
        status: res.status
      });

    default:
      throw new _errors.CraftAiUnknownError({
        more: parseBody(req, resBody).message,
        request: req,
        status: res.status
      });
  }
}

function createHttpAgent() {
  var proxy = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : undefined;

  if (_constants.IN_BROWSER) {
    return undefined;
  } else if (proxy) {
    var HttpProxyAgent = require('http-proxy-agent');

    return new HttpProxyAgent(proxy);
  } else {
    var http = require('http');

    return new http.Agent({
      keepAlive: true
    });
  }
}

function createHttpsAgent() {
  var proxy = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : undefined;

  if (_constants.IN_BROWSER) {
    return undefined;
  } else if (proxy) {
    var HttpsProxyAgent = require('https-proxy-agent');

    return new HttpsProxyAgent(proxy);
  } else {
    var https = require('https');

    return new https.Agent({
      keepAlive: true
    });
  }
}

function createRequest(cfg) {
  var defaultHeaders = {
    'Authorization': "Bearer ".concat(cfg.token),
    'Content-Type': 'application/json; charset=utf-8',
    'Accept': 'application/json'
  };

  if (!_constants.IN_BROWSER) {
    // Don't set the user agent in browsers it can cause CORS issues
    // e.g. Safari v10.1.2 (12603.3.8)
    defaultHeaders['User-Agent'] = USER_AGENT;
  }

  var baseUrl = "".concat(cfg.url, "/api/v1/").concat(cfg.owner, "/").concat(cfg.project);
  var agent = baseUrl.slice(0, 5) === 'https' ? createHttpsAgent(cfg.proxy) : createHttpAgent(cfg.proxy);
  return function (req) {
    req = _.defaults(req || {}, {
      agent: agent,
      method: 'GET',
      path: '',
      body: undefined,
      query: {},
      headers: {}
    });
    req.url = req.url || "".concat(baseUrl).concat(req.path);
    var queryStr = Object.entries(req.query).filter(function (_ref) {
      var _ref2 = _slicedToArray(_ref, 2),
          key = _ref2[0],
          value = _ref2[1];

      return !_.isUndefined(value);
    }).map(function (keyVal) {
      return keyVal.join('=');
    }).join('&');

    if (queryStr.length > 0) {
      req.url += "?".concat(queryStr);
    }

    req.headers = _.defaults(req.headers, defaultHeaders);
    req.body = req.body && JSON.stringify(req.body);
    return fetch(req.url, req)["catch"](function (err) {
      debug("Network error while executing ".concat(req.method, " ").concat(req.path), err);
      return Promise.reject(new _errors.CraftAiNetworkError({
        more: err.message
      }));
    }).then(function (res) {
      return res.text()["catch"](function (err) {
        debug("Invalid response from ".concat(req.method, " ").concat(req.path), err);
        throw new _errors.CraftAiInternalError('Internal Error, the craft ai server responded an invalid response, see err.more for details.', {
          request: req,
          more: err.message
        });
      }).then(function (resBody) {
        return parseResponse(req, res, resBody);
      });
    });
  };
}