"use strict";

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.tzFromOffset = tzFromOffset;
exports["default"] = Time;

var _errors = require("./errors");

var _momentMini = _interopRequireDefault(require("moment-mini"));

var _ = _interopRequireWildcard(require("./lodash"));

var _timezones = _interopRequireWildcard(require("./timezones"));

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

// From 'moment/src/lib/parse/regex.js'
var OFFSET_REGEX = /Z|[+-]\d\d:?\d\d/gi; // +00:00 -00:00 +0000 -0000 or Z

function tzFromOffset(offset) {
  if (_.isInteger(offset)) {
    var sign = offset >= 0 ? '+' : '-';
    var abs = Math.abs(offset); // If the offset belongs to [-15, 15] it is considered to represent hours
    // This reproduces Moment's utcOffset behaviour.

    if (abs < 16) {
      return "".concat(sign).concat(_.padStart(abs, 2, '0'), ":00");
    }

    return "".concat(sign).concat(_.padStart(Math.floor(abs / 60), 2, '0'), ":").concat(_.padStart(abs % 60, 2, '0'));
  } else {
    return offset;
  }
}

function Time() {
  var t = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : undefined;
  var tz = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : undefined;

  // Make sure it works with or without new.
  if (!(this instanceof Time)) {
    return new Time(t, tz);
  }

  var m;

  if (t instanceof Time) {
    // t is an instance of Time
    m = _momentMini["default"].unix(t.timestamp);
    m.utcOffset(t.timezone);
  } else if (_momentMini["default"].isMoment(t)) {
    // t is an instance of moment
    m = t;
  } else if (_.isDate(t)) {
    // t is a js Date
    m = (0, _momentMini["default"])(t);
  } else if (_.isNumber(t)) {
    // t is a posix timestamp
    // it might be a float as retrieved by Date.now() / 1000
    m = _momentMini["default"].unix(t);
  } else if (_.isString(t)) {
    // To make it consistent everywhere we only handle ISO 8601 format
    if (t.match(OFFSET_REGEX)) {
      // String with a explicit offset
      m = _momentMini["default"].parseZone(t, _momentMini["default"].ISO_8601);
    } else {
      // No explicit offset
      m = (0, _momentMini["default"])(t, _momentMini["default"].ISO_8601);
    }
  } else if (_.isUndefined(t)) {
    m = (0, _momentMini["default"])();
  }

  if (m === undefined || !m.isValid()) {
    throw new _errors.CraftAiTimeError("Time error, given \"".concat(t, "\" is invalid."));
  }

  if (tz) {
    // tz formats should be parseable by moment
    if (!(0, _timezones["default"])(tz)) {
      throw new _errors.CraftAiTimeError("Time error, the given timezone \"".concat(tz, "\" is invalid.\n      Please refer to the client's documentation to see accepted formats:\n      https://beta.craft.ai/doc/http#context-properties-types."));
    }

    m.utcOffset(_timezones.timezones[tz] || tz);
  }

  var minuteOffset = m.utcOffset();
  return _.extend(this, {
    timestamp: m.unix(),
    timezone: tzFromOffset(minuteOffset),
    time_of_day: m.hour() + m.minute() / 60 + m.second() / 3600,
    day_of_month: m.date(),
    // we want day to be in [1;31]
    month_of_year: m.month() + 1,
    // we want months to be in [1;12]
    day_of_week: m.isoWeekday() - 1,
    // we want week day to be in [0;6]
    utc: m.toISOString()
  });
}